@@ -1,5 +1,5 @@
 poisson.iarCAR <-
-function(formula, beta=NULL, phi=NULL, tau2=NULL, W, burnin=0, n.sample=1000, blocksize.beta=5, blocksize.phi=10, prior.mean.beta=NULL, prior.var.beta=NULL, prior.max.tau2=NULL)
+function(formula, beta=NULL, phi=NULL, tau2=NULL, W, burnin=0, n.sample=1000, thin=1, blocksize.beta=5, blocksize.phi=10, prior.mean.beta=NULL, prior.var.beta=NULL, prior.max.tau2=NULL)
 {
 ##############################################
 #### Format the arguments and check for errors
@@ -109,6 +109,8 @@ offset <- try(model.offset(frame), silent=TRUE)
     if(n.sample <= 0) stop("n.sample is less than or equal to zero.", call.=FALSE)
     if(burnin < 0) stop("burn-in is less than zero.", call.=FALSE)
     if(n.sample <= burnin)  stop("Burn-in is greater than n.sample.", call.=FALSE)
+    if(!is.numeric(thin)) stop("thin is not a number", call.=FALSE)
+    if(thin <= 0) stop("thin is less than or equal to zero.", call.=FALSE)
 
     if(!is.numeric(blocksize.beta)) stop("blocksize.beta is not a number", call.=FALSE)
     if(blocksize.beta <= 0) stop("blocksize.beta is less than or equal to zero", call.=FALSE)
@@ -175,10 +177,11 @@ offset <- try(model.offset(frame), silent=TRUE)
 
 
 ## Matrices to store samples
-samples.beta <- array(NA, c((n.sample-burnin), p))
-samples.phi <- array(NA, c((n.sample-burnin), n))
-samples.tau2 <- array(NA, c((n.sample-burnin), 1))
-samples.deviance <- array(NA, c((n.sample-burnin), 1))
+n.keep <- floor((n.sample - burnin)/thin)
+samples.beta <- array(NA, c(n.keep, p))
+samples.phi <- array(NA, c(n.keep, n))
+samples.tau2 <- array(NA, c(n.keep, 1))
+samples.deviance <- array(NA, c(n.keep, 1))
 
 ## Metropolis quantities
 accept.all <- rep(0,4)
@@ -221,7 +224,8 @@ tau2.posterior.shape <- 0.5 * n - 1
     if(ncol(W)!= n) stop("W has the wrong number of columns.", call.=FALSE)
     if(sum(is.na(W))>0) stop("W has missing 'NA' values.", call.=FALSE)
     if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
-    if(!sum(names(table(W))==c(0,1))==2) stop("W has non-binary (zero and one) values.", call.=FALSE)
+    if(min(W)<0) stop("W has negative elements.", call.=FALSE)
+    if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
 
 n.neighbours <- as.numeric(apply(W, 1, sum))
 Q <- diag(n.neighbours)  - W
@@ -328,9 +332,9 @@ block.var.chol <- as.list(rep(0,n.phi.block))
     ###################
     ## Save the results
     ###################
-        if(j > burnin)
+        if(j > burnin & (j-burnin)%%thin==0)
         {
-        ele <- j - burnin
+        ele <- (j - burnin) / thin
         samples.beta[ele, ] <- beta
         samples.phi[ele, ] <- phi
         samples.tau2[ele, ] <- tau2
@@ -353,10 +357,10 @@ block.var.chol <- as.list(rep(0,n.phi.block))
         accept <- c(0,0,0,0)
             
         #### beta tuning parameter
-            if(accept.beta > 40)
+            if(accept.beta > 70)
             {
             proposal.sd.beta <- 2 * proposal.sd.beta
-            }else if(accept.beta < 30)              
+            }else if(accept.beta < 50)              
             {
             proposal.sd.beta <- 0.5 * proposal.sd.beta
             }else
@@ -397,6 +401,12 @@ block.var.chol <- as.list(rep(0,n.phi.block))
 ###################################
 #### Summarise and save the results 
 ###################################
+## Acceptance rates
+accept.beta <- 100 * accept.all[1] / accept.all[2]
+accept.phi <- 100 * accept.all[3] / accept.all[4]
+accept.tau2 <- 100
+
+
 ## Deviance information criterion (DIC)
 median.beta <- apply(samples.beta, 2, median)
 median.phi <- apply(samples.phi, 2, median)
@@ -404,43 +414,47 @@ fitted.median <- exp(X.standardised %*% median.beta + median.phi + offset)
 deviance.fitted <- -2 * sum(Y * log(fitted.median) -  fitted.median - lfactorial(Y))
 p.d <- mean(samples.deviance) - deviance.fitted
 DIC <- 2 * mean(samples.deviance) - deviance.fitted
-residuals <- Y - fitted.median
 
 
 
 #### transform the parameters back to the origianl covariate scale.
 samples.beta.orig <- samples.beta
-    for(r in 1:p)
-    {
-        if(X.indicator[r]==1)
-        {
-        samples.beta.orig[ ,r] <- samples.beta[ ,r] / X.sd[r]
-        }else if(X.indicator[r]==2 & p>1)
-        {
-        X.transformed <- which(X.indicator==1)
-        samples.temp <- as.matrix(samples.beta[ ,X.transformed])
-            for(s in 1:length(X.transformed))
-            {
-            samples.temp[ ,s] <- samples.temp[ ,s] * X.mean[X.transformed[s]]  / X.sd[X.transformed[s]]
-            }
-        intercept.adjustment <- apply(samples.temp, 1,sum) 
-        samples.beta.orig[ ,r] <- samples.beta[ ,r] - intercept.adjustment
-        }else
-        {
-        }
-    }
-
+number.cts <- sum(X.indicator==1)     
+     if(number.cts>0)
+     {
+          for(r in 1:p)
+          {
+               if(X.indicator[r]==1)
+               {
+               samples.beta.orig[ ,r] <- samples.beta[ ,r] / X.sd[r]
+               }else if(X.indicator[r]==2 & p>1)
+               {
+               X.transformed <- which(X.indicator==1)
+               samples.temp <- as.matrix(samples.beta[ ,X.transformed])
+                    for(s in 1:length(X.transformed))
+                    {
+                    samples.temp[ ,s] <- samples.temp[ ,s] * X.mean[X.transformed[s]]  / X.sd[X.transformed[s]]
+                    }
+               intercept.adjustment <- apply(samples.temp, 1,sum) 
+               samples.beta.orig[ ,r] <- samples.beta[ ,r] - intercept.adjustment
+               }else
+               {
+               }
+          }
+     }else
+     {
+     }
 
 
 #### Create a summary object
 samples.beta.orig <- mcmc(samples.beta.orig)
 summary.beta <- t(apply(samples.beta.orig, 2, quantile, c(0.5, 0.025, 0.975))) 
-summary.beta <- cbind(summary.beta, rep((n.sample-burnin), p), as.numeric(100 * (1-rejectionRate(samples.beta.orig))))
+summary.beta <- cbind(summary.beta, rep(n.keep, p), rep(accept.beta,p))
 rownames(summary.beta) <- colnames(X)
 colnames(summary.beta) <- c("Median", "2.5%", "97.5%", "n.sample", "% accept")
 
 summary.hyper <- quantile(samples.tau2, c(0.5, 0.025, 0.975))
-summary.hyper <- c(summary.hyper, (n.sample-burnin), as.numeric(100 * (1-rejectionRate(mcmc(samples.tau2)))))
+summary.hyper <- c(summary.hyper, n.keep, accept.tau2)
 
 summary.results <- rbind(summary.beta, summary.hyper)
 rownames(summary.results)[nrow(summary.results)] <- "tau2"
@@ -461,21 +475,30 @@ random.effects <- round(random.effects, 4)
 
 #### Create the Fitted values
 fitted.values <- array(NA, c(n, 5))
+residuals <- array(NA, c(n, 5))
 colnames(fitted.values) <- c("Mean", "Sd", "Median", "2.5%", "97.5%")
+colnames(residuals) <- c("Mean", "Sd", "Median", "2.5%", "97.5%")
 fitted.temp <- array(NA, c(nrow(samples.beta), n))
-
+residuals.temp <- array(NA, c(nrow(samples.beta), n))
     for(i in 1:nrow(samples.beta))
     {
-    fitted.temp[i, ] <- exp(X.standardised %*% samples.beta[i, ] + samples.phi[i, ] + offset)
+    temp <- exp(X.standardised %*% samples.beta[i, ] + samples.phi[i, ] + offset)
+    fitted.temp[i, ] <- temp
+    residuals.temp[i, ] <- Y - temp
     }
 fitted.values[ ,1] <- apply(fitted.temp, 2, mean)
 fitted.values[ ,2] <- apply(fitted.temp, 2, sd)
 fitted.values[ ,3:5] <- t(apply(fitted.temp, 2, quantile, c(0.5, 0.025, 0.975)))
 fitted.values <- round(fitted.values, 4)
+residuals[ ,1] <- apply(residuals.temp, 2, mean)
+residuals[ ,2] <- apply(residuals.temp, 2, sd)
+residuals[ ,3:5] <- t(apply(residuals.temp, 2, quantile, c(0.5, 0.025, 0.975)))
+residuals <- round(residuals, 4)
 
 
 
 #### Print a summary of the results to the screen
+## Compile and return the results
 cat("\n#################\n")
 cat("#### Model fitted\n")
 cat("#################\n\n")
@@ -491,11 +514,11 @@ cat("############\n\n")
 cat("Posterior quantiles and acceptance rates\n\n")
 print(summary.results)
 cat("\n\n")
-cat("Acceptance rate for the random effects is ", round(100 * accept.all[3] / accept.all[4],1), "%","\n\n", sep="")
+cat("Acceptance rate for the random effects is ", round(accept.phi,1), "%","\n\n", sep="")
 cat("DIC = ", DIC, "     ", "p.d = ", p.d, "\n")
 
 
-## Compile and return the results
 results <- list(formula=formula, samples.beta=samples.beta.orig, samples.phi=mcmc(samples.phi), samples.tau2=mcmc(samples.tau2), fitted.values=fitted.values, random.effects=random.effects, residuals=residuals, DIC=DIC, p.d=p.d, summary.results=summary.results)
 return(results)
 }
+
